/*
 *   Copyright 2011 Institute of Communication and Computer Systems (ICCS) 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */

/**
 * @file   allocator.h
 * @author Ioannis Koutras (joko@microlab.ntua.gr)
 * @date   August 2012
 * @brief  Basic structures needed for the dmmlib allocator.
 */

#ifndef ALLOCATOR_H
#define ALLOCATOR_H
#include "dmm_config.h"
#include "dmmlib/raw_block.h"

#ifdef HAVE_LOCKS
#include <pthread.h>
#endif /* HAVE_LOCKS */

#ifdef WITH_ALLOCATOR_STATS
#include <dmmlib/dmmstats.h>
#endif /* WITH_ALLOCATOR_STATS */

/** The allocator structure of dmmlib. */
typedef struct allocator_s {
    raw_block_header_t *raw_block_head; /**< The head of the raw blocks list. */
#ifdef HAVE_LOCKS
    pthread_mutex_t creation_mutex; /**< Mutex to allow the creation of new raw
                                      blocks. */
#endif /* HAVE_LOCKS */
#ifdef WITH_ALLOCATOR_STATS
    dmmstats_t dmm_stats;
#endif /* WITH_ALLOCATOR_STATS */
} allocator_t;

#endif /* ALLOCATOR_H */
