/*
 *   Copyright 2012 Institute of Communication and Computer Systems (ICCS) 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */

/**
 * @file   bitmap.h
 * @author Ioannis Koutras (joko@microlab.ntua.gr)
 * @brief  Memory allocation for bitmap-organized raw blocks
 */

#ifndef BITMAP_H
#define BITMAP_H
#include "dmm_config.h"

#include "dmmlib/raw_block.h"

#ifdef BITMAP_RB_ONLY
#define dmmlib_malloc(raw_block, size) bitmap_malloc(raw_block, size)
#define dmmlib_free(raw_block, size) bitmap_free(raw_block, size)
#endif /* BITMAP_RB_ONLY */

void * bitmap_malloc(raw_block_header_t *raw_block, size_t size);

void bitmap_free(raw_block_header_t *raw_block, void *ptr);

void * bitmap_realloc(raw_block_header_t *raw_block, void * ptr,
        size_t req_size);

#endif /* BITMAP_H */
