/*
 *   Copyright 2011 Institute of Communication and Computer Systems (ICCS) 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */

/**
 * \file        lifo_order.h
 * \author      Ioannis Koutras (joko@microlab.ntua.gr)
 * \date        January, 2012
 *  
 * \brief Add a block in a LIFO-ordered linked list.
 */

#ifndef LIFO_ORDER_H
#define LIFO_ORDER_H
#include "dmm_config.h"
#include "freelist/freelist_rb.h"
#include "freelist/block_header.h"

#ifdef LIFO_SORT_POLICY
#define add_block(raw_block, block) add_block_lifo_order(raw_block, block)
#endif /* LIFO_SORT_POLICY */

/**
 * Adds a block in a LIFO-ordered list.
 *
 * @param raw_block The pointer to the raw block.
 * @param block The pointer of the block to be added.
 */
void add_block_lifo_order(
        freelist_rb_t *raw_block,
        block_header_t *block);

#endif /* LIFO_ORDER_H */

