/*
 *   Copyright 2012 Institute of Communication and Computer Systems (ICCS) 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */

/**
 * @file   bitmap_realloc.c
 * @author Ilias Pliotas, Ioannis Koutras
 * @date   September, 2012
 * @brief  realloc() implementation for bitmap-organized raw blocks
 */

#include "bitmap/bitmap.h"
#include "bitmap/bitmap_rb.h"
#include "dmmlib/dmmlib.h"

#include <string.h>

/**
 * Reallocates a memory block from a bitmap-organized raw block
 *
 * @param  raw_block The pointer of the raw block.
 * @param  ptr       The pointer of the memory block to be re-allocated.
 * @param  req_size  The requested memory size.
 * @retval           The address to serve the request.
 * @retval NULL      No available memory space.
 */
void * bitmap_realloc(raw_block_header_t *raw_block, void * ptr,
        size_t req_size) { 
    void *ret;
    bitmap_rb_t *rb_header;
    chunk_header_t *chunk_header;

    rb_header = (bitmap_rb_t *)((char *)raw_block + sizeof(raw_block_header_t));
    chunk_header = (chunk_header_t *)((char *)ptr - CHUNK_HDR_SIZE);

    ret = malloc(req_size);
    ret = memmove(ret, ptr,
            chunk_header->num_of_cells * rb_header->bytes_per_cell);
    bitmap_free(raw_block, ptr);
    return ret;
}
