/*
 *   Copyright 2011 Institute of Communication and Computer Systems (ICCS) 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */

#include <stddef.h>
#include "linked_lists/add_block_in_order.h"
#include "linked_lists/linked_lists.h"

#ifdef COUNT_HOPS
void add_block_in_order(
        heap_t *heap,
        void **block,
        void **starting_node,
        bool (*search_condition)(void *block, void* comparing_block)) {
#else /* COUNT_HOPS */
void add_block_in_order(
        void **block,
        void **starting_node,
        bool (*search_condition)(void *block, void* comparing_block)) {
#endif /* COUNT_HOPS */
    void *current_node;
    bool added;
#ifdef BLOCKS_IN_SLL
    void *previous_node;

    previous_node = NULL;
#endif /* BLOCKS_IN_SLL */

    added = false;

    /* if the list is empty, put the block in the list's head */
    if(*starting_node == NULL) {
        *starting_node = *block;
        set_next(*block, NULL);
#ifdef BLOCKS_IN_DLL
        set_previous(*block, NULL);
#endif /* BLOCKS_IN_DLL */
    } else {
        /* traverse the list to find the right spot to put the block */
        for(current_node = *starting_node; current_node != NULL;
                current_node = get_next(current_node)) {
#ifdef COUNT_HOPS
            heap->dmm_stats.total_hops++;
#endif /* COUNT_HOPS */
            if(search_condition(*block, current_node) == true) {
#ifdef BLOCKS_IN_SLL
                if(previous_node != NULL) {
                    set_next(previous_node, *block);
                }
#else /* BLOCKS_IN_SLL */
                if(get_previous(current_node) != NULL) {
                    set_next(get_previous(current_node), *block);
                }
                set_previous(*block, get_previous(current_node);
                set_previous(current_node, *block);
#endif /* BLOCKS_IN_SLL */
                set_next(*block, current_node);
                added = true;
                break;
            }
            previous_node = current_node;
        }
        /* in case no spot was found, put the block last */
        if(added == false) {
            set_next(previous_node, *block);
#ifdef BLOCKS_IN_DLL
            set_previous(*block, previous_node);
#endif /* BLOCKS_IN_DLL */
        }
    }
}

