/*
 *   Copyright 2011 Institute of Communication and Computer Systems (ICCS) 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */

#include "freelist/linked_lists/linked_lists.h"

#ifdef BLOCKS_IN_DLL

void set_previous(block_header_t *ptr, block_header_t *previous_block) {
    ptr->previous = previous_block;
}

void * get_previous(void *ptr) {
    return (void *) ptr->previous;
}

#endif /* BLOCKS_IN_DLL */

#ifdef BLOCKS_IN_SLL
void remove_block(block_header_t *block, freelist_rb_t *raw_block) {

    block_header_t *current_node, *previous_node;

    previous_node = NULL;

    /* Traverse a list starting from the starting node until block is found. */
    for(current_node = raw_block->free_list_head; current_node != NULL;
            current_node = current_node->next) {
        if(current_node == block) {
            if(current_node == raw_block->free_list_head) {
                raw_block->free_list_head = block->next;
            } else {
                previous_node->next = block->next;
            }
            break;
        }
        previous_node = current_node;
    }
}
#endif /* BLOCKS_IN_SLL */

#ifdef BLOCKS_IN_DLL
void remove_block(void **block, void **starting_node) {
    void *previous_block, *next_block;

    /* No need to traverse the list, just check if the block is the starting
     * node of the list.
     */

    previous_block = get_previous(*block);
    next_block = get_next(*block);

    if(*block == *starting_node) {
        *starting_node = previous_block;
    }

    if(previous_block != NULL) {
        set_next(previous_block, next_block);
    }

    if(next_block != NULL) {
        set_previous(next_block, previous_block);
    }

}
#endif /* BLOCKS_IN_DLL */
