/*
 *   Copyright 2011 Institute of Communication and Computer Systems (ICCS) 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */

#include "dmmlib/raw_block.h"

#ifdef HAVE_LOCKS
#include <pthread.h>
#endif /* HAVE_LOCKS */
#include "request_memory.h"
#ifdef FL_RB_ONLY
#include "freelist/freelist_rb.h"
#endif /* FL_RB_ONLY */
#ifdef BITMAP_RB_ONLY
#include "bitmap/bitmap_rb.h"
#endif /* BITMAP_RB_ONLY */

raw_block_header_t *create_new_raw_block(size_t raw_block_size, rb_type type) {
    void *ptr;
#ifdef FL_RB_ONLY
    freelist_rb_t *fl_rb;
#endif /* FL_RB_ONLY */
#ifdef BITMAP_RB_ONLY
    bitmap_rb_t *bitmap_rb;
    BMAP_EL_TYPE *bitmap_p;
#endif /* BITMAP_RB_ONLY */

    ptr = request_memory(raw_block_size);

    if(ptr == NULL) {
        return NULL;
    }

    ((raw_block_header_t *)ptr)->size = raw_block_size;
    ((raw_block_header_t *)ptr)->next_raw_block = NULL;

    switch(type) {

#ifdef FL_RB_ONLY
        case FREELIST:
            fl_rb = (freelist_rb_t *)((char *)ptr +
                    sizeof(raw_block_header_t));
            fl_rb->border_ptr = (block_header_t *)((char *)fl_rb +
                    sizeof(freelist_rb_t));
            fl_rb->free_list_head = NULL;
            break;
#endif /* FL_RB_ONLY */

#ifdef BITMAP_RB_ONLY
        case BITMAP:
            bitmap_rb = (bitmap_rb_t *)((char *)ptr +
                    sizeof(raw_block_header_t));
            bitmap_rb->bytes_per_cell = BITMAP_RESOLUTION;

            bitmap_rb->elements =
                (raw_block_size + BMAP_EL_SIZE - 
                 sizeof(raw_block_header_t) - sizeof(bitmap_rb_t)) /
                (BMAP_EL_SIZE + BMAP_EL_SIZE_BITS * bitmap_rb->bytes_per_cell);

            bitmap_p = &bitmap_rb->bmap;
            for(size_t i = 0; i < bitmap_rb->elements; ++i) {
                *bitmap_p = BMAP_EL_INIT_VAL;
                bitmap_p++;
            }
            break;
#endif /* BITMAP_RB_ONLY */
        case BIGBLOCK:
            break;
    }

#ifdef HAVE_LOCKS
    pthread_mutex_init(&((raw_block_header_t *)ptr)->mutex, NULL);
#endif /* HAVE LOCKS */

    return (raw_block_header_t *)ptr;
}
